#!/usr/bin/python

#####################################################################################
# pico_status_hv4.0.py
# Version 1.0 Relase Date 01.09.2022
# Script to show you some statistics pulled from your UPS PIco HV4.0
# -*- coding: utf-8 -*-
# re-Written by Ioannis A. Mourtsiadis info@pimodules.com
#####################################################################################


# Set your desired temperature symbol
# C = Celsius
# F = Fahrenheit
degrees = "C"

'''
   unsigned int8     backing_mode;			 ///1   (0x01 b)   // Backup Mode (SCAP or Battery or Mixed)
   unsigned int16    rtime;                  ///2   (0x02 w)   // Available Running Time on Battery powering / Battery Energy Level
   unsigned int16    pico_is_running;        ///2   (0x04 b)   // PIco and RPi is running     
   unsigned int16    curilevel;              ///2   (0x0e w)   // Current Input Level
   unsigned int16    curolevel;              ///2   (0x10 w)   // Current output Level
   unsigned int16    ecilevel;               ///2   (0x12 w)   // EPR Current Input Level
   unsigned int16    aEXT0level;             ///2   (0x14 w)   // A/D 5V0 Voltage Level
   unsigned int16    aEXT1level;             ///2   (0x16 w)   // A/D 5V0 Voltage Leve
   unsigned int8     key_pressed;            ///1   (0x18 b)   // Keys Pressed Status
   unsigned int16    optlevel;         		 ///2   (0x1c w)   // Opto Couple level or state if digital
   unsigned int8     BAT_chg_stat;           ///1   (0x1e b)   // Charger Status
   unsigned int8     BAT_chg_real;           ///1   (0x1f b)   // Charger Status real  
   unsigned int8     SCA_pwm_stat;           ///1   (0x20 b)   // Charger Status  
   unsigned int8     FAN_status;             ///1   (0x21 b)   // FAN Status (ON/OFF or PWM)
   unsigned int16    sysinfo;                ///2   (0x22 w)   // System Information
   unsigned int8     RS232_status;           ///1   (0x24 b)   // RS232 Status
   unsigned int8     RS232_data[16];         ///1   (0x25 b)   // RS232 Received Data 
  
  	
'''
import sys
import time
import datetime
import os
import re     
import getopt
import smbus
#import psutil
import time
import datetime
import threading
import RPi.GPIO as GPIO

i2c = smbus.SMBus(1)

############### FUNCTIONS #################

def fw_version():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x38)
   data = format(data,"04x")
   return data
   
def pcb_version():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x69, 0x35)
   data = format(data,"02x")
   return data   
   
def PIco_model():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x69, 0x36)
   data = format(data,"02x")
   return data
   
def pcb_battery():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x69, 0x37)
   if (data == 0x4C):
      return "Battery LiPO 'L'	0x4C"
   elif (data == 0x49):
      return "Battery Li-Ion 'I'	0x49"
   elif (data == 0x46):
      return "Battery LiFePO4 'F'	0x46"
   elif (data == 0x48):
      return "Battery NiMH 'H'	0x48"
   elif (data == 0x41):
      return "Battery SAL 'A'	0x41"
   elif (data == 0x43):
      return "On Board Super Cap (100F) 'C' 0x43"
   elif (data == 0x44):
      return "External Super Cap HAT 'D' (300F-500F-800F) 0x44"
   elif (data == 0x6C):
      return "Battery LiPO+SC 'l' 0x6C"
   elif (data == 0x69):
      return "Battery Li-Ion+SC	'i'	0x69"
   elif (data == 0x66):
      return "Battery LiFePO4+SC 'f' 0x66"
   elif (data == 0x68):
      return "Battery NiMH+SC 'h' 0x68"
   elif (data == 0x61):
      return "Battery SAL+SC 'a' 0x61"
   else:
      return "ERROR" 
  
def default_battery():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x07)
   if (data == 0x4C):
      return "Battery LiPO 'L'	0x4C"
   elif (data == 0x49):
      return "Battery Li-Ion 'I'	0x49"
   elif (data == 0x46):
      return "Battery LiFePO4 'F'	0x46"
   elif (data == 0x48):
      return "Battery NiMH 'H'	0x48"
   elif (data == 0x41):
      return "Battery SAL 'A'	0x41"
   elif (data == 0x43):
      return "On Board Super Cap (100F) 'C' 0x43"
   elif (data == 0x44):
      return "External Super Cap HAT 'D' (300F-500F-800F) 0x44"
   elif (data == 0x6C):
      return "Battery LiPO+SC 'l' 0x6C"
   elif (data == 0x69):
      return "Battery Li-Ion+SC	'i'	0x69"
   elif (data == 0x66):
      return "Battery LiFePO4+SC 'f' 0x66"
   elif (data == 0x68):
      return "Battery NiMH+SC 'h' 0x68"
   elif (data == 0x61):
      return "Battery SAL+SC 'a' 0x61"
   else:
      return "ERROR"
      
def getCPUtemperature():
    time.sleep(0.1)
    res = os.popen('vcgencmd measure_temp').readline()
    cpu_data=res.replace("temp=","").replace("'C\n","")
    cpu_int=int(float(cpu_data))
    return format(cpu_int,"02d")
    
def getFANThreshold():
   data = i2c.read_byte_data(0x6b, 0x13)
   data = format(data,"02x")
   return data 
    
def fan_state():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x69, 0x21)
   data = format(data,"02d")
   if (data == 1):
      return "ON"
   elif (data == 0):
      return "OFF"
   else:
      return data
      
def pwr_mode():
   data = i2c.read_byte_data(0x69, 0x00)
   data = data & ~(1 << 7)
   if (data == 1):
      return "Cable POWERED"
   elif (data == 0):
      return "BAT/SCAP POWERED"
   else:
      return "ERROR"
      
def backing_mode():
   data = i2c.read_byte_data(0x69, 0x01)
   #data = data & ~(1 << 7)
   if (data == 0x01):
      return "Battery Backup"
   elif (data == 0x02):
      return "Super Capacitor Backup"
   elif (data == 0x03):
      return "Mixed Bat/SCap Backup"       
   else:
      return "ERROR"
   
def sca_read():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x06)
   data = format(data,"02x")
   return (float(data) / 100) 

def bat_read():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x08)
   data = format(data,"02x")
   return (float(data) / 100) 

def rpi_read():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x0a)
   data = format(data,"02x")
   return (float(data) / 100) 

def epr_read():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x0c)
   data = format(data,"02x")
   return (float(data) / 100)
   
def rpi_hold_on():
   time.sleep(0.1)
   data = i2c.read_word_data(0x6b, 0x03)
   data = format(data,"02x")
   return data   
 
def bat_runtime():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x01)
   if (data == 0xff):
      return "Unlimited - Timer Disabled"      
   else:
      return data

def relay_state():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x0b)
   if (data == 0x01):
      return "ON"      
   else:
      return "OFF"
      
def RS232_rate():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x02)
   if (data == 0x00):
      return "RPI_RS232_OFF"
   elif (data == 0x01):
      return "RPI_RS232_4800" 
   elif (data == 0x02):
      return "RPI_RS232_9600"
   elif (data == 0x03):
      return "RPI_RS232_19200"
   elif (data == 0x04):
      return "RPI_RS232_38400"
   elif (data == 0x05):
      return "RPI_RS232_57600"
   elif (data == 0x0f):
      return "RPI_RS232_115200"
   else:
      return "ERROR"
      

def sta_runtime():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x05)
   if (data == 0xff):
      return "Timer Disabled"      
   else:
      return data

def fssd_duration():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x1a)
   return data  
   
def enable5V():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x06)
   if (data == 0x01):
      return "ON"      
   else:
      return "OFF"

def on_the_go():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x16)
   if (data == 0xaa):
      return "Activated"      
   else:
      return "de-Activated"  

def rpi_OFF_time_reg():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x1b)
   return data  
      
def pico_is_running():
   time.sleep(0.1)
   data0 = i2c.read_word_data(0x69, 0x04)
   time.sleep(0.5)
   data1 = i2c.read_word_data(0x69, 0x04)
   if(data0 == data1):
      return "Faulty - Not Running"
   else:
      return "Healthy - Running Properly"
      
def ad0_read():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x14)
   data = format(data,"02x")
   return (float(data) / 100)
   
def ad1_read():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x16)
   data = format(data,"02x")
   return (float(data) / 100)
   

print (" ")
print ("***********************************")
print ("      UPS PIco HV4.0 Status        ")
print ("Version 1.0  Relase Date 01.09.2022")
print ("***********************************")
print (" ")
print (" ","UPS PIco Firmware...............:",fw_version())
print (" ","UPS PIco Bootloader.............:",pcb_version())
print (" ","UPS PIco PCB Version............:",PIco_model())
print (" ","UPS PIco PCB Defaut Battery.....:",pcb_battery())
print (" ","UPS PIco Set Battery............:",default_battery())
print (" ")
print (" ","Raspberry Pi Core Temprature....:",int(getCPUtemperature(),10), "C")
print (" ","Embedded Fan Temp. Threshold....:",getFANThreshold(), "C")
print (" ","Embedded Fan Speed..............:",fan_state(),"% of Total RPM")
print (" ")
print (" ","Powering Mode...................:",pwr_mode())
print (" ","Power Backup Set Mode...........:",backing_mode())
print (" ")
print (" ","Battery Level...................:",bat_read(), "V")
print (" ","Super Capacitor Level...........:",sca_read(), "V")
print (" ","EPR/PPoE Powering Level.........:",epr_read(), "V")
print (" ","RPi(USB) Powering Level.........:",rpi_read(), "V")
print (" ","Running Time on Battery.........:",bat_runtime(), "min")
#print (" ","Current Source of Power Backup..:",backup_source())
print (" ")
print (" ","Startup on  Hold time...........:",rpi_hold_on(),"Seconds")
print (" ","FSSD Durtion....................:",fssd_duration(),"Seconds")
print (" ","Firmware loop...................:",pico_is_running())
print (" ","Still Alive Timer...............:",sta_runtime())
print (" ","Raspberry Pi OFF waiting time...:",rpi_OFF_time_reg(),"min")
print (" ")
print (" ","UPS PIco HV4.0 RS232 Setup......:",RS232_rate())
print (" ","Internal Relay State............:",relay_state())
print (" ","5V supply during LPR............:",enable5V())
print (" ","Magic Switch Funtionality.......:",on_the_go())

   
#   unsigned int8    led_green;               /// (0x09 b)     GREEN LED ON/OFF
#   unsigned int8    led_blue;                /// (0x0a b)     BLUE LED ON/OFF
#   unsigned int8    map_led_green;           /// (0x17 b)  
#   unsigned int8    map_led_blue;            /// (0x18 b)   
   
#   unsigned int8    chg_ctrl;                /// (0x19)     // Charger funtionality,
#   unsigned int8    LED_OFF;                 /// (0x15 b)    LEDs ON =0, LEDS OFF =1;
  



print (" ","A/D Converter 0 ................:",ad0_read())
print (" ","A/D Converter 1 ................:",ad1_read())
print (" ")
print ("***********************************")
print ("     Powered by PiModules          ")
print ("***********************************")
print (" ")